#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int	apply_patches(unsigned char **ppbin, unsigned long *pbin_size, unsigned long patchseg_size,
						unsigned long *pLOWCODE_base, unsigned long *pHIGHCODE_base);

int
main(int argc, char *argv[])
{
	FILE	*fh_in, *fh_out;
	char	*fname_in, *fname_out;
	unsigned char	*pbin;
	unsigned long	filesize;
	unsigned long	patchseg_size;
	unsigned long	LOWCODE_base, HIGHCODE_base;

	if (argc != 4) {
		fprintf(stderr, "Usage: %s patchseg_size infile outfile\n", argv[0]);
		exit(1);
	}

	patchseg_size = strtoul(argv[1], (char **)0, 0);
	if (!patchseg_size) {
		fprintf(stderr, "Invalid patch segment size!\n");
		exit(1);
	}
	fname_in = argv[2];
	fname_out = argv[3];

	if (!(fh_in = fopen(fname_in, "rb"))) {
		fprintf(stderr, "Cannot open input file '%s'!\n", fname_in);
		exit(1);
	}

	fseek(fh_in, 0L, SEEK_END);
	filesize = ftell(fh_in);
	fseek(fh_in, 0L, SEEK_SET);

	if (!(pbin = (unsigned char *)malloc(filesize))) {
		fprintf(stderr, "Error allocating buffer!\n");
		fclose(fh_in);
		exit(1);
	}

	if (fread(pbin, 1, filesize, fh_in) != filesize) {
		fprintf(stderr, "Error reading input file!\n");
		fclose(fh_in);
		free(pbin);
		exit(1);
	}
	fclose(fh_in);


	if (!apply_patches(&pbin, &filesize, patchseg_size, &LOWCODE_base, &HIGHCODE_base)) {
		fprintf(stderr, "Error applying patches!\n");
		free(pbin);
		exit(1);
	}

	if (!(fh_out = fopen(fname_out, "wb"))) {
		fprintf(stderr, "Cannot create output file '%s'!\n", fname_out);
		free(pbin);
		exit(1);
	}

	if (fwrite(pbin, 1, filesize, fh_out) != filesize) {
		fprintf(stderr, "Error writing output file!\n");
		fclose(fh_out);
		unlink(fname_out);
		free(pbin);
		exit(1);
	}

	fclose(fh_out);

	printf("LOWCODE_BASE\tequ\t0x%x\n", LOWCODE_base);
	printf("HIGHCODE_BASE\tequ\t0x%x\n", HIGHCODE_base);
	printf("PATCHSEG_SIZE\tequ\t0x%x\n", patchseg_size);

	return 0;
}


#define	OBJHDR_V_SIZE	2
#define	OBJHDR_V_ADDR	3
#define	OBJHDR_P_SIZE	4
#define	OBJHDR_P_ADDR	5
#define	OBJHDR_FLAGS	9

int
apply_patches(unsigned char **ppbin, unsigned long *pbin_size, unsigned long patchseg_size,
			  unsigned long *pLOWCODE_base, unsigned long *pHIGHCODE_base)
{
	unsigned char	*pbin = *ppbin;
	unsigned long	bin_size = *pbin_size;
	unsigned long	pe_header_offset;
	unsigned char	*pe_hdr;
	unsigned short	num_objects;
	unsigned short	nt_header_size;
	unsigned char	*object_table_base;
	unsigned long	*object_header, *prev_object_header;
	int	i;

	if ((pbin[0] != 'M') || (pbin[1] != 'Z')) {
		fprintf(stderr, "Invalid EXE header!\n");
		return 0;
	}

	pe_header_offset = *(unsigned int *)(pbin + 0x3c);
	if (pe_header_offset > bin_size) {
		fprintf(stderr, "Invalid EXE header!\n");
		return 0;
	}

	pe_hdr = pbin + pe_header_offset;
	if ((pe_hdr[0] != 'P') || (pe_hdr[1] != 'E')) {
		fprintf(stderr, "Invalid PE header!\n");
		return 0;
	}
	num_objects = *(unsigned short *)(pe_hdr + 6);
	nt_header_size = *(unsigned short *)(pe_hdr + 20);

	object_table_base = (pe_hdr + 24 + nt_header_size);

	object_header = (unsigned long *)object_table_base;
	prev_object_header = (unsigned long *)0;
	for (i = 0; i < num_objects; i++) {
		if (!strcmp((void *)object_header, "INIT")) {
			break;
		}
		prev_object_header = object_header;
		object_header += 10;
	}

	if (i == num_objects) {
		fprintf(stderr, "Couldn't find INIT section!\n");
		return 0;
	}
	if (!prev_object_header) {
		fprintf(stderr, "INIT section is first section - can't deal with that!\n");
		return 0;
	}
	if (strncmp((void *)prev_object_header, "IDEXPRDT", 8)) {
		fprintf(stderr, "WARNING: 'IDEXPRDT' section doesn't precede INIT section\n");
	}

#if 0
	printf("PREV seg:\n\tv_size=0x%8.8x, rva=0x%8.8x\n\tp_size=0x%8.8x, p_ofs=0x%8.8x\n\tflags=0x%8.8x\n",
		prev_object_header[OBJHDR_V_SIZE], prev_object_header[OBJHDR_V_ADDR], 
		prev_object_header[OBJHDR_P_SIZE], prev_object_header[OBJHDR_P_ADDR], prev_object_header[OBJHDR_FLAGS]);

	printf("INIT seg:\n\tv_size=0x%8.8x, rva=0x%8.8x\n\tp_size=0x%8.8x, p_ofs=0x%8.8x\n\tflags=0x%8.8x\n",
		object_header[OBJHDR_V_SIZE], object_header[OBJHDR_V_ADDR], 
		object_header[OBJHDR_P_SIZE], object_header[OBJHDR_P_SIZE], object_header[OBJHDR_FLAGS]);

#endif

	// Adjust PE header's "image size" to include patch segment
	*((unsigned long *)(pe_hdr + 80)) += patchseg_size;
	
	// Adjust physical size of previous seg
	prev_object_header[OBJHDR_P_SIZE] += patchseg_size;
	// Set virtual size of previous seg to same as physical size
	prev_object_header[OBJHDR_V_SIZE] = prev_object_header[OBJHDR_P_SIZE];
	// Clear 'initialized data segment' flag for previous seg
	prev_object_header[OBJHDR_FLAGS] &= ~0x40;
	// Set 'executable', 'readable', 'writeable', 'code' flags for previous seg
	prev_object_header[OBJHDR_FLAGS] |= 0xE0000020;

	// Calc address of "LOWCODE"
	*pLOWCODE_base = (prev_object_header[3] + prev_object_header[2] - patchseg_size) + 0x80010000;

	// Adjust physical offset of INIT seg
	object_header[OBJHDR_P_ADDR] += patchseg_size;
	// Adjust virtual address of INIT seg
	object_header[OBJHDR_V_ADDR] += patchseg_size;

	// Calc address of "HIGHCODE"
	*pHIGHCODE_base = (object_header[OBJHDR_V_ADDR] + object_header[OBJHDR_P_SIZE] - patchseg_size) + 0x80010000;

	if (!(pbin = realloc(pbin, bin_size + patchseg_size))) {
		fprintf(stderr, "Error allocating space for patch segment!\n");
		return 0;
	}

	memset(pbin + bin_size, 0x50, patchseg_size);

	*ppbin = pbin;
	*pbin_size = (bin_size + patchseg_size);

	return 1;
}
